function [ccoef,triangles,triples] = clustering_coefficient(A,max_n,seed, output_array)
  % Estimate the clustering coefficient of a graph.
  % 
  % The global clustering coefficient is defined as:
  %  C = number of closed triples / number of triples of nodes
  % number_of_triples = #paths_of_length_2 - paths that loop back
  %                   = #paths2 - #edge
  
  % For large graphs, sample n,m nodes
  if nargin<2
    max_n = 2e5;
  end
  if nargin<3
    seed = 12345;
  end
  srand(seed);
  n = min(max_n,length(A));
  m = min(n+1,  length(A)); % for catching errors, use different sizes
  
  c = randperm(length(A),n);
  d = randperm(length(A),m);
  cd = intersect(c,d);
  
  A1 = A(c,d); % edges from i∈c to j∈d
  A2 = A(c,:)*A(:,d); % paths of length 2 from i∈c to j∈d
  A3 = A1.*A2; % triangles with i∈c, j∈d, k∈A
  
  % exclude self-loops or back-and-forth paths
  % if i∈c and i∈d, then A2(i,i) shouldn't be counted  (with indices adjusted).
  % its value will be sum(A(:,i).^2)
  if length(cd) == 0
    A2overcount = 0;
    A3overcount = 0;
  else
    A2overcount = full(sum(sum(A(:,cd).^2)));
    A3overcount = full(sum(diag(A(cd,cd))' .* sum(A(:,cd).^2)));
  end
  
  paths2 = sum(sum(A2)) - A2overcount; % number of paths of length 2 that don't loop back
  paths3 = sum(sum(A3)) - A3overcount;
  
  triangles = paths3 / 6;
  triples = (paths2-paths3) / 2 + paths3 / 6;
  
  %ccoef = paths21 / (paths2 + eps);
  ccoef = triangles / (triples + eps);
  
  if nargin>=4 && output_array
    ccoef = [ccoef;triangles;triples];
  end
end

