function [A,labels,nodepos] = load_pajek(file,useWeights)
	%
	% Author: Scott White
	% Creates a n x n sparse weighted adjacency matrix from a pajek file
	% Note: this function does not support mixed graphs where both edges
	% and arcs are defined in the same file. Furthermore, only one edge
	% can be defined per line.
	%
	% INPUTS:
	%       file: name of the pajek file to load
	%       useWeighs: optional parameter to specify whether or not to use
	%       weights
	%
	% OUTPUTS:
	%       A: n x n sparse weighted adjacency matrix
	%       labels: labels associated with each vertex and matrix row
	if ~ischar(file)
	   error('Specify file name as a string');
	end
	
	[fid,message] = fopen(file,'rt');
	if fid == -1
		fprintf('%s',message);
		error('File could not be opened');
	end
	
	lineCtr = 0;
	numV = fscanf(fid,'*Vertices %d \r\n');
	lineCtr = lineCtr + 1;
	labels = {};
	for i=1:numV
		line = fgetl(fid);
		lineCtr = lineCtr + 1;
		vId = sscanf(line,'%d');
		% Read label
		pos = strfind(line,'"');
		if length(pos) == 0
			label = sprintf('%d',vId);
		else
			label = line(pos(1)+1:pos(2)-1);   
			pos2 = strfind(label,' ');
			if length(pos2) == 2
				label = label(pos2(end):length(label)); 
			end
		end
		labels{i} = label;
		% Read position
		if pos(2)+1 < length(line)
		  nodepos(i,:) = sscanf(line(pos(2)+1:end),'%f');
		end
		% done
		if mod(i,10000) == 0
			fprintf('%d vertices loaded...\n',i);
		end
	end
	
	line = fgetl(fid);
	line = lower(line);
	pos = strfind(line,'arc');
	directed = 0;
	if length(pos) > 0
		directed = 1;
	end
	edges = zeros(numV*5,3);
	edgeCtr = 1;
	while 1
		line = fgetl(fid);
		values = sscanf(line,'%d %d %f');
		lineCtr = lineCtr + 1;
		if length(values) > 3
			values = values(1:3,:);
		end
		if length(values) < 2
			fprintf('Stopping on line %d...\nExpeting at least 2 values and not %d.\n',lineCtr,length(values));
			break;
		elseif length(values) == 2 || ( nargin == 2 && useWeights == 0)
			values(3) = 1;
		end
		if values(1) == values(2)
			%fprintf('Skipping self-loop at edge line %d...\n',i);
			continue;
		end
		if values(1) < 1 || values(2) < 1 || values(1) > numV || values(2) > numV
			fprintf('Error on edge line %d. Ids out of bounds\n',edgeCtr);
			continue;
		end
		edges(edgeCtr,:) = values';
		edgeCtr = edgeCtr+1;
		if feof(fid)
			%fprintf('End of file reached.\n');
			break;
		end    
		if mod(edgeCtr,10000) == 0
			fprintf('%d edges loaded...\n',edgeCtr);
		end
	end
	edges = unique(edges,'rows');
	numEdges = length(edges);
	if edges(1,1) == 0
		edges = edges(2:numEdges,:);
		numEdges = numEdges - 1;
	end
	if directed == 0
		otherEdges = zeros(numEdges,3);
		otherEdges(:,1) = edges(:,2);
		otherEdges(:,2) = edges(:,1);
		otherEdges(:,3) = edges(:,3);
		edges(numEdges+1:2*numEdges,:) = otherEdges;
		edges = unique(edges,'rows');
		numEdges = length(edges);
		if edges(1,1) == 0
			edges = edges(2:numEdges,:);
			numEdges = numEdges - 1;
		end
	end
	
	A = spconvert(edges);
	%numV = fscanf(fid,'*Vertices %g \r\n');

	%fprintf(fid,'%g  "v%g" \r',[vertices; vertices]);
	%fprintf(fid,'*Arcs \r');
	%fprintf(fid,'%g  %g  %g\r',arcs);
	fclose(fid);
end
