% A matlab port of https://gist.github.com/dgleich/cf170a226aa848240cf4
% 
% Heat Kernel Based Community Detection 
% by Kyle Kloster and David F. Gleich
% 
% Port by Twan van Laarhoven
% 
% This might be slightly different from the hkgrow mex implementation
function [c,bestcond] = optimize_cluster_heat_kernel(A,cSeed)
  seed = find(cSeed);
  degree = full(sum(A))';

  %% Setup parameters that can be computed automatically
  N = 47; % see paper for how to set this automatically
  t = 15.;
  eps = 0.0001;
  % compute_psis(N,t), note: 1 based indexing
  psis = zeros(N+1,1);
  psis(N+1) = 1.;
  for i = N-1:-1:0
    psis(i+1) = psis(i+1+1)*t / (i+1) + 1;
  end
  
  %% Estimate hkpr vector 
  % 
  % This is our main algorithm.
  if 0
    % Direct port of python code
    
    % t, tol, N, psis are precomputed
    % npush tracks number of operations
    x = sparse(length(A),1); % Store x as sparse vector
    r = sparse(length(A),N); % initialize residual
    npush = 0.;
    % initialize queue
    % since matlab doesn't have queues, we use two pointers into a matrix
    % The range Q_data(:,Q_head:Q_tail-1) is valid
    Q_data = zeros(2,max(1000,length(seed)));
    Q_head = 1; % reading position
    Q_tail = 1; % inserting position
    
    for s = seed
      r(s,0+1) = 1./length(seed);
      assert(Q_tail <= size(Q_data,2));
      Q_data(:,Q_tail) = [s;0];
      Q_tail = Q_tail+1;
    end
    npush = npush + length(seed);
    
    while Q_head < Q_tail
      % pop from queue
      v = Q_data(1,Q_head);
      j = Q_data(2,Q_head);
      Q_head = Q_head + 1;
      rvj = r(v,j+1);
      % perform the hk-relax step
      x(v) = x(v) + rvj;
      r(v,j+1) = 0;
      update = rvj / degree(v);
      mass = (t / (j+1)) * update;
      % for neighbors of v
      neighbors = find(A(:,v));
      for u = neighbors(:)'
        assert(numel(u)==1);
        if j+1 == N
          x(u) = x(u) + update;
          continue
        end
        thresh = exp(t)*eps*degree(u);
        thresh = thresh/(N*psis(j+1+1));
        rnext = r(u,j+1+1);
        if rnext < thresh && rnext + mass >= thresh
          % Add to queue
          if Q_tail > size(Q_data,2)
            % We either grow the queue or just refill it
            Q_data_old = Q_data;
            Q_head_old = Q_head;
            Q_tail_old = Q_tail;
            if Q_head < size(Q_data,2)/2
              Q_data = zeros(2, 2*size(Q_data,2));
            end
            Q_head = 1;
            Q_tail = Q_head + (Q_tail_old - Q_head_old);
            Q_data(:,Q_head:(Q_tail-1)) = Q_data_old(:,Q_head_old:(Q_tail_old-1));
            clear Q_data_old Q_head_old Q_tail_old;
          end
          assert(Q_tail <= size(Q_data,2));
          Q_data(:,Q_tail) = [u;j+1];
          Q_tail = Q_tail+1;
        end
        r(u,j+1+1) = rnext + mass;
      end
      if npush>100000
        npush
        rnext
        mass
        j
        nnz(x)
        break
      end
      npush = npush + degree(v);
    end
  else
    % Vectorized vesion
    x = sparse(length(A),1); % Store x as sparse vector
    r = cSeed ./ length(seed); % r in iteration j
    for j = 0:N
      v = find(r);
      update = r(v) ./ degree(v);
      mass = (t / (j+1)) * update;
      x = x + r;
      % push mass
      if j+1 == N
        x = x + A(:,v) * update;
        break;
      end
      r = A(:,v) * mass;
      % threshold
      u = find(r);
      thresh = (exp(t) * eps ./ (N*psis(j+1+1))) * degree(u);
      r(u) = r(u) .* (r(u) >= thresh);
    end
  end
  
  %% Step 2 do a sweep cut based on this vector 
  % Find cluster, first normalize by degree
  x = x ./ (degree+eps);

  % now sort x's keys by value, decreasing
  xi = find(x);
  [sx,px] = sort(x(xi),'descend');
  sv = xi(px);
  S = [];
  volS = 0.;
  cutS = 0.;
  Gvol = sum(degree);
  bestcond = 1.;
  bestset = [x];
  for s = sv(:)'
    assert(numel(s)==1);
    volS = volS + degree(s); % add degree to volume
    cutS = cutS + degree(s) - 2*full(sum(A(S,s)));
    S(end+1) = s;
    if cutS/(min(volS,Gvol-volS)+eps) < bestcond
      bestcond = cutS/(min(volS,Gvol-volS)+eps);
      bestset = S;
    end
  end
  
  % the best cluster
  c = sparse(bestset,1,1,length(A),1);
end

