function [cBest,pBest] = optimize_cluster_mh(A,d,cSeed,cInit,opts)
  % Optimize probabilistic model with Metropolis Hastings sampler.
  % Return MAP result
  % 
  % Probabilistic model:
  %  P( a_{ij} | d_i, d_j, c_i, c_j) = 1 - (1 - α_{c_i+c_j})^(d_i * d_j)^ρ
  % Parameters:
  %  ρ,α₀,α₁,α₂
  
  % model parameters
  if ~isfield(opts,'rho') opts.rho = 1; end
  if ~isfield(opts,'p0')  opts.p0  = 0.01; end % alpha above
  if ~isfield(opts,'p1')  opts.p1  = opts.p0; end
  if ~isfield(opts,'p2')  opts.p2  = 0.1; end
  % sampling parameters
  if ~isfield(opts,'min_iter')         opts.min_iter = 1000; end
  if ~isfield(opts,'iter_times_best')  opts.iter_times_best = 3; end
  if ~isfield(opts,'add_only')         opts.add_only = false; end
  if ~isfield(opts,'seed_term')        opts.seed_term = true; end
  if ~isfield(opts,'prb_add_neighbor') opts.prb_add_neighbor = 0.8; end
  if ~isfield(opts,'prb_add_any')      opts.prb_add_any = 0.1; end
  if ~isfield(opts,'prb_del')          opts.prb_del = 0.1; end
  opts.max_iter = 10000;
  
  % 
  dRho = d.^opts.rho;
  sum_dRho = sum(dRho);
  sum_dRho2 = sum(dRho.^2);
  volume = sum(d);
  
  logP0 = log1p(-opts.p0); % log(1-opts.p0)
  logP1 = log1p(-opts.p1); % log(1-opts.p1)
  logP2 = log1p(-opts.p2); % log(1-opts.p2)
  
  % initial
  c = cInit;
  edges = A*c;
  neighbors = edges > 0;
  ci = find(c);
  neighbors(ci) = 0;
  
  % log probability.
  % split into two parts:
  %  pAll   for non-edges between all pairs of nodes, calculated from sum_dRho
  %  pEdge  adjestment for edges. calculated per edge.
  %         We skip the edges outside c, since that is just a constant
  %         instead we count negative this constant term for internal edges
  sum_dRhoC  = sum(dRho(ci));
  sum_dRhoC2 = sum(dRho(ci).^2);
  pAll = 0 ...%logP0 * 0.5 * (sum_dRho^2-sum_dRho2) ... % constant term
       + (logP1-logP0) * (sum_dRho-sum_dRhoC) * sum_dRhoC ...
       + (logP2-logP0) * 0.5 * (sum_dRhoC^2-sum_dRhoC2);
  % for edges
  % Note: we should not count within edges twice
  [is,js,ws] = find(A(:,ci));
  duplicate = c(is)==1 & is>js;
  is(duplicate) = []; js(duplicate) = [];
  is_w = is(c(is)==1); js_w = js(c(is)==1);
  is_b = is(c(is)==0); js_b = js(c(is)==0);
  pEdges = sum(log1mexp_diff(logP2*(d(is_w).*d(js_w)).^opts.rho)) ...
         + sum(log1mexp_diff(logP1*(d(is_b).*d(js_b)).^opts.rho)) ...
         - sum(log1mexp_diff(logP0*(d(is  ).*d(js  )).^opts.rho));
  p = pAll + pEdges - opts.seed_term*log(nnz(c));
  
  cBest = c;
  pBest = p;
  
  iter = 0;
  best_iter = 0; % iteration in which c=cBest was set
  
  for iter = 1:opts.max_iter
    if iter > opts.min_iter && iter > opts.iter_times_best * best_iter
      break;
    end
    
    % propose a move
    u = rand();
    if u < opts.prb_add_neighbor || opts.add_only
      % add a neighboring node
      ni = find(neighbors);
      i = ni(randperm(length(ni),1));
      new_c = c;
      new_c(i) = 1;
      new_edges = edges + A(:,i);
      new_sum_dRhoC  = sum_dRhoC  + dRho(i);
      new_sum_dRhoC2 = sum_dRhoC2 + dRho(i).^2;
      js = find(A(:,i));
      js_w = js(c(js)==1);
      js_b = js(c(js)==0);
      new_pEdges = pEdges + sum(log1mexp_diff(logP2*(d(i)*d(js_w)).^opts.rho) ...
                              - log1mexp_diff(logP1*(d(i)*d(js_w)).^opts.rho)) ...
                          + sum(log1mexp_diff(logP1*(d(i)*d(js_b)).^opts.rho) ...
                              - log1mexp_diff(logP0*(d(i)*d(js_b)).^opts.rho));
      if opts.add_only
        pMove    = 1;
        pInverse = 1;
      else
        pMove    = 1/nnz(neighbors); % probability of this move
        pInverse = 1/(nnz(new_c)-nnz(cSeed)); % probability of the inverse move
      end
    %elseif u - opts.prb_add_neighbor < opts.prb_add_any
    %  % add any node
    %  pMove = opts.prb_add_any / (opts
    
    else
      % remove a node at random
      % we require that the remaining cluster is connected
      removed = 0;
      for trying=1:10
        i = ci(randperm(length(ci),1));
        if cSeed(i), continue; end % don't remove seeds
        new_c = c;
        new_c(i) = 0;
        % check connectedness
        new_ci = find(new_c);
        cc = connected_components(A(new_ci,new_ci));
        if max(cc) > 1, continue; end
        % score
        new_edges = edges - A(:,i);
        new_sum_dRhoC  = sum_dRhoC  - dRho(i);
        new_sum_dRhoC2 = sum_dRhoC2 - dRho(i).^2;
        js = find(A(:,i));
        js_w = js(c(js)==1);
        js_b = js(c(js)==0);
        new_pEdges = pEdges - sum(log1mexp_diff(logP2*(d(i)*d(js_w)).^opts.rho) ...
                                - log1mexp_diff(logP1*(d(i)*d(js_w)).^opts.rho)) ...
                            - sum(log1mexp_diff(logP1*(d(i)*d(js_b)).^opts.rho) ...
                                - log1mexp_diff(logP0*(d(i)*d(js_b)).^opts.rho));
        pMove    = 1/(nnz(c)-nnz(cSeed));
        pInverse = 1/nnz(neighbors); % should use new_neighbors
        removed = 1;
        break;
      end
      if ~removed, continue; end
    end
    
    % calculate new loss
    new_pAll = 0 ...%logP0 * 0.5 * (sum_dRho^2-sum_dRho2) ...
             + (logP1-logP0) * (sum_dRho-sum_dRhoC) * new_sum_dRhoC ...
             + (logP2-logP0) * 0.5 * (new_sum_dRhoC^2-new_sum_dRhoC2);
    new_p = new_pAll + new_pEdges - opts.seed_term*log(nnz(new_c));
    accepted = log(rand()) < new_p - p - log(pMove) + log(pInverse);
    
    if opts.verbose >= 2
      fprintf('Propose %f (%d)\n',new_p,nnz(new_c));
      new_pAll
      new_pEdges
    end
    % accept?
    if accepted
      if opts.verbose >= 1
        fprintf('Accept %f (%d)\n',new_p,nnz(new_c));
        new_c
      end
      c          = new_c;
      ci         = find(c);
      pAll       = new_pAll;
      pEdges     = new_pEdges;
      p          = new_p;
      edges      = new_edges;
      neighbors  = edges > 0; neighbors(ci) = 0;
      sum_dRhoC  = new_sum_dRhoC;
      sum_dRhoC2 = new_sum_dRhoC2;
      if p > pBest
        cBest = c;
        pBest = p;
        best_iter = iter;
      end
    end
  end
end

function y = log1mexp(x)
  y = log1p(-exp(x)); % Accurate for large |x|
  %y = log(-expm1(x)); % Accurate for small |x|, better when |x| < 1
end
function y = log1mexp_diff(x)
  y = log1mexp(x) - x;
end
