function run_methods_on_datasets(optimizer_args)
  % Run all methods on all datasets, with all different seed set sizes, and different repetitions
  % write the summary results to .mat files
  
  if nargin<1
    optimizer_args = struct();
  end
  
  more off;
  
  if exist('OCTAVE_VERSION','builtin')
    include_big = nproc > 10; % only use big datasets on big machines
  elseif exist('feature','builtin');
    include_big = feature('numCores') > 10;
  else
    include_big = 1;	
  end
  
  datasets = datasets_for_experiments('include_big',include_big);
  methods = methods_for_experiments();
  seeds = seeds_for_experiments();
  max_cluster = 1000;
  
  if ~exist('OCTAVE_VERSION','builtin')
    time = @cputime;
  end
  
  for is = 1:numel(seeds)
    seed = seeds{is};
    for id = 1:numel(datasets)
      dataset = datasets{id};
      % skip (for performance reasons?)
      if isfield(seed,'skip_data') && ismember(dataset.name, seed.skip_data)
        continue;
      end
      % repetitions with different random seeds, different random subset of clusters, differen random graphs (for lfr graphs)
      for rep = 1:dataset.num_reps
        data_loaded = false;
        clear A C CSub;
        clear R;
        
        for im = 1:numel(methods)
          method = methods{im};
          
          % skip?
          if isfield(method,'skip_data') && ismember(dataset.name, method.skip_data)
            continue;
          end
          
          % Have we already done this experiment?
          filename = output_file_for_experiment(method, dataset, seed, rep);
          if exist(filename,'file')
            if rep == dataset.num_reps
              fprintf('[%s] Skipping %s\n',datestr(now,'HH:MM:SS'),filename);
            end
            continue; % TODO: check if this file has the right size at least
          end
          
          % Load data
          if ~data_loaded
            fprintf('[%s] Loading %s\n',datestr(now,'HH:MM:SS'),dataset.name);
            data_loaded = 1;
            [A,C] = load_dataset(dataset, rep*10);
            % Run experiments on a random subset of the clusters, and with random seeds
            if dataset.num_dups > 1
              C = repmat(C,1,dataset.num_dups);
            end
            if size(C,2) >= max_cluster
              srand(rep*10+1);
              CSub = C(:,randperm(size(C,2),max_cluster));
            else
              CSub = C;
            end
            
            % Pick seed nodes
            if isequal(seed.type,'geornd')
              [R,C] = random_seeds_geornd(C, seed.size, rep*10+2);
            elseif isequal(seed.type,'fixed')
              C = CSub;
              R = random_seeds(C, seed.size, rep*10+2);
            else
              error('Unknown seed type: %s', seed.type);
            end
          end
          
          % Run method
          fprintf('[%s] Running %s  (%d,%d)\n',datestr(now,'HH:MM:SS'),filename,size(C,1),size(C,2));
          start_time = time();
          opts = method;
          fn = fieldnames(optimizer_args);
          for i=1:length(fn)
            opts.(fn{i}) = optimizer_args.(fn{i});
          end
          opts.truth = C;
          opts.output_summary = 1;
          out = optimize_cluster(A,R, opts);
          
          out.method  = method;
          out.rep     = rep;
          out.seeds   = seed;
          out.dataset = dataset;
          out.dataset_vertices = length(A);
          out.dataset_edges    = nnz(A)/2;
          out.dataset_clusters = size(R,2);
          out.total_runtime = time() - start_time;
          save('-v7',filename,'out');
          clear out;
        end
      end
    end
  end
end

