function [r,dr,l]=score_conductance_size(A,c,n,d)
  % conductance score, where cluster size is limited to n
  
  if nargin<4 || isempty(d)
    d = full(sum(A));
  end
  % Ensure that input is within bounds (as best as we can)
  if true
    c = max(0,min(c,1));
    c = c * (n / (sum(c)+eps));
    c = max(0,min(c,1));
  end
  
  % significantly faster to do the sparse multiplication manually
  [i,unused,k] = find(c);
  if length(k)<10000
    w = full(diag( c'*(A(:,i)*k) ))';
  else
    w = full(diag( k'*(A(i,i)*k) ))';
  end
  v = full(d*c);
  v = v+eps; % prevent division by 0
  r = w/v;
  
  if nargout >= 2
    % gradient, see score_conductance
    dw = 2*A(:,i)*k; % faster multiplication
    dv = d';
    
    % Test if the sparse gradient would be enough
    if 1
      relevant = union(find(dw),find(c));
      dr_r = full(dw(relevant)/v - dv(relevant)*(w/v^2));
      c_r = full(c(relevant));
      active_r = (c_r>0 & c_r<1) | (c_r==0 & dr_r >= 0) | (c_r==1 & dr_r <= 0);
      if sum(dr_r(active_r)) >= 0
        % fast search
        %[l,active_r] = find_lagrange_multiplier_sort(c_r,dr_r);
        [l,active_r] = find_lagrange_multiplier(c_r,dr_r);
        dr_r = dr_r - l;
        dr = sparse(relevant,1,active_r.*dr_r,length(c),1);
        return;
      end
    end
    
    % Calculate the full gradient
    dr = dw/v - dv*(w/v^2);
    dr = full(dr);
    
    % Return dr-l, where l is lagrange multiplier
    % use iterative solver to find l such that
    %   ∑{i s.t. c_i∉{0,1} | c_i=0&dr_i-λ≤0 | c_i=1&dr_i-λ≥0 }(dr_i-λ) = 0
    [l,active]=find_lagrange_multiplier(c,dr);
    % subtract from gradient
    dr = dr - l;
    % restrict to active set
    dr = dr .* active;
    dr = sparse(dr);
  end
end

function [l,active]=find_lagrange_multiplier(c,dr)
  if 0
    [l,active] = find_lagrange_multiplier_iterative(c,dr,100);
  elseif 0
    [l,active] = find_lagrange_multiplier_annealed(c,dr,low,mid,high);
  elseif 0
    [l,active] = find_lagrange_multiplier_sort(c,dr,low,mid,high);
  else
    % try iterative solver first, revert to sort based solver if that fails
    [l,active] = find_lagrange_multiplier_iterative(c,dr, 10);
    if ~isnan(l), return; end
    %warning('Using sorting solver');
    [l,active] = find_lagrange_multiplier_sort(c,dr);
  end
  if isnan(l)
    error('Lagrange multiplier not converged');
  end
end

function [l,active] = find_lagrange_multiplier_iterative(c,dr, maxit)
  % simple iterative solver
  l = 0;
  positive_l_only = 0; % we already check for the l>=0 case earlier
  if positive_l_only
    [i,unused,k] = find(c);
    mid_i  = i(k>0 & k<1);
    high_i = i(k==1);
    low_positive_i = setdiff(find(dr>0),i);
  else
    mid  = full(c>0&c<1);
    low  = full(c==0);
    high = full(c==1);
  end
  
  for it=1:maxit
    if ~positive_l_only
      active = mid | (low&(dr>=l)) | (high&(dr<=l));
    else
      active = mid_i;
      if l >= 0
        % In the common case that l>=0 we don't need to look at everything, just the nodes in c, and those with gradient > 0
        active = [active; low_positive_i(dr(low_positive_i) >= l)];
      else
        l = nan; return; % Perhaps just give up otherwise
        low_i = find(c==0);
        active = [active; low_i(dr(low_i) >= l)];
      end
      active = [active; high_i(dr(high_i) <= l)];
    end
    dr_active  = sum(dr(active));
    cnt_active = nnz(active);
    l1 = dr_active / (cnt_active + eps);
    if abs(l1-l) <= 1e-10*abs(l) % l1==l
      if positive_l_only
        active = sparse(active,1,1,length(c),1);
      end
      return;
    elseif mod(it,10)==0
      a = 1-1/(1+it/10);
      l = (a*l+(1-a)*l1); % smoother steps
    else
      l = l1;
    end
  end
  l = NaN;
end

function [l,active] = find_lagrange_multiplier_annealed(c,dr,low,mid,high)
  % smooth iterative search
  l = 0;
  ls = [];
  sigmoid = @(x)1./(1+exp(-x));
  %hardness = 5 ./ mean(abs(dr)); % increasing hardness
  hardness = 1e8;
  for it=1:100
    active = mid + low.*sigmoid(hardness*(dr-l)) + high.*sigmoid(hardness*(l-dr));
    dr_active  = sum(dr.*active);
    cnt_active = sum(active);
    l1 = dr_active / (cnt_active + eps);
    ls = [ls;l1];
    if l1==l
      break;
    else
      l = l1;
      hardness = hardness * 2;
    end
  end
end

function [l,active] = find_lagrange_multiplier_sort(c,dr,low,mid,high)
    % Sorting based search: lambda must fall somewhere between two adjacen ∇f values, consider them all.
    if nargin < 5
      mid  = full(c>0&c<1);
      low  = full(c==0);
      high = full(c==1);
    end
    [drs,i] = sort(full(dr));
    cnt_ub  = low(i);
    cnt_lb  = high(i);
    cnt_mid = mid(i);
    drs_cum = [0;cumsum(drs.*cnt_lb)] + flipud([0;cumsum(flipud(drs.*cnt_ub))]) + sum(drs.*cnt_mid);
    cnt_cum = [0;cumsum(     cnt_lb)] + flipud([0;cumsum(flipud(     cnt_ub))]) + sum(     cnt_mid);
    % what would the optimal lambda be given that it is in this interval?
    lambdas = drs_cum./(cnt_cum+eps);
    % is that optimal lambda in the interval itself? If it is, then it is a valid solution.
    valid = find([-1e10;drs]<=lambdas+eps & lambdas-eps<=[drs;1e10]);
    assert(length(valid)>=1); % we need at least one solution. Multiple solutions are possible, but they will be equal
    l = lambdas(valid(1));
    active = mid | (low&(dr>=l)) | (high&(dr<=l));
end

