function write_datasets_tables(varargin)
  opts = struct(varargin{:});
  if ~isfield(opts,'outdir'), opts.outdir = '../tables'; end
  if ~isfield(opts,'parallel'), opts.parallel = 3; end
  
  which_datasets = {'include_big',1, 'include_5k',0, 'include_5k_combined',0};
  
  more off;
  fprintf('Calculating dataset statistics\n');
  datasets = datasets_for_experiments(which_datasets{:});
  datasets = cellfun(@(x)dataset_statistics(x,opts), datasets, 'UniformOutput',false);
  
  write_datasets_table(datasets, sprintf('%s/table-datasets.tex',opts.outdir), 0);
  write_datasets_table(datasets, sprintf('%s/table-datasets-combined.tex',opts.outdir), 1);
end

function dataset = dataset_statistics(dataset,opts)
  % Do stats exist already?
  filename = sprintf('../out/dataset-%s.mat',dataset.name);
  if (exist(filename))
    d = dataset;
    load(filename,'dataset');
    dataset.display_name    = d.display_name;
    dataset.display_variant = d.display_variant;
    return;
  end
  
  if ~exist('OCTAVE_VERSION','builtin')
    fflush = @(stream)drawnow('update');
    stdout = 0;
  end
  
  % Load dataset
  fprintf('For %s\n  ',dataset.name);fflush(stdout);
  [A,C] = load_dataset(dataset, 1*10);
  assert(nnz(A) == nnz(A|A')); % Test: symmetric matrix
  assert(nnz(diag(A)) == 0); % Test: no self-loops
  
  % Calculate
  dataset.num_vertices = length(A);
  dataset.num_edges    = nnz(A)/2;
  dataset.num_clusters = size(C,2);
  dataset.cluster_size = full(sum(C));
  dataset.mean_cluster_size = mean(dataset.cluster_size);
  dataset.median_cluster_size = median(dataset.cluster_size);
  fprintf('.');fflush(stdout);
  % within cluster edges
  %dataset.mean_within = mean(cluster_within(A,C));
  %fprintf('.');fflush(stdout);
  
  % conductance
  d = full(sum(A));
  if size(C,1) < 100
    cond = score_conductance(A,C,d);
  else
    cond = pararrayfun_compat(opts.parallel, @(i)score_conductance(A,C(:,i),d), 1:size(C,2), 'ChunksPerProc',100);
  end
  dataset.conductance = cond;
  dataset.mean_conductance = mean(cond);
  fprintf('.');fflush(stdout);
  
  % clustering coefficient (estimate)
  %clustering_coefficient(A);
  coef = pararrayfun_compat(opts.parallel,@(seed)clustering_coefficient(A,2e5,seed,1),12345:(12345+100));
  est_coef = sum(coef(2,:)) ./ (sum(coef(3,:)) + eps);
  dataset.clustering_coefficient_parts = coef;
  dataset.clustering_coefficient = est_coef;
  fprintf('.');fflush(stdout);
  
  % random cluster statistics
  max_cluster = 5000;
  rep = 0;
  which = randperm(size(C,2),min(size(C,2),max_cluster));
  if length(which) < max_cluster
    which = repmat(which,1,ceil(max_cluster/length(which)));
  end
  % Note: using ChunksPerProc for random_cluster will fail unless we have 'UniformOutput'=false
  % but then we would get back a cell array
  CR = pararrayfun_compat(opts.parallel,@(i)random_cluster(A,nnz(C(:,which(i))),10*rep+i),1:length(which));
  cond = pararrayfun_compat(opts.parallel, @(i)score_conductance(A,CR(:,i),d), 1:size(CR,2), 'ChunksPerProc',10);
  dataset.conductance_random = cond;
  dataset.mean_conductance_random = mean(cond);
  fprintf('.\n');fflush(stdout);
  
  % Save
  save('-v7',filename,'dataset');
end

function write_datasets_table(datasets, filename, combined)
  % Number of variants to put horizontally
  if combined
    variants = {};
    for id=1:numel(datasets)
      dataset = datasets{id};
      if isfield(dataset,'display_variant') && ~ismember(dataset.display_variant, variants)
        variants = [variants,dataset.display_variant];
      end
    end
  else
    variants = {''};
  end
  
  include_conductance_random = 0;
  
  % Write table
  fprintf('Writing %s\n',filename);
  file = fopen(filename,'wt');
  
  fprintf(file,'\\begin{tabular}{lrrr');
  for iv=1:numel(variants)
    fprintf(file,'@{\\hspace*{5mm}}r@{\\hspace*{3mm}}r@{\\hspace*{3mm}}r');
    if include_conductance_random
      fprintf(file,'@{\\hspace*{3mm}}r');
    end
  end
  fprintf(file,'}\n');
  
  fprintf(file,'\\hlinetop\n');
  
  if combined
    fprintf(file,'&&&');
    for iv=1:numel(variants)
      if include_conductance_random
        fprintf(file,'&\\multicolumn{4}{c}{%s}', variants{iv});
      else
        fprintf(file,'&\\multicolumn{3}{c}{%s}', variants{iv});
      end
    end
    fprintf(file,'\\\\\n');
  end
  
  fprintf(file,'Dataset & \\#node & \\#edge & clus.c.');
  for iv=1:numel(variants)
    fprintf(file,' & \\#comm & $\\overline{|\\setc|}$ & $\\overline{\\score(\\setc)}$');
    if include_conductance_random
      fprintf(file,' & $\\overline{\\score(R)}$');
    end
  end
  fprintf(file,'\\\\\n');
  
  fprintf(file,'\\hlinemid\n');
  
  same_row = 0;
  for id = 1:numel(datasets)
    dataset = datasets{id};
    % first of row?
    if ~same_row
	    fprintf(file,'%% %s\n',dataset.name);
	    fprintf(file,'%-11s & %7d & %8d & %.3f', ...
	      dataset.display_name, ...
	      dataset.num_vertices, ...
	      dataset.num_edges, ...
	      dataset.clustering_coefficient);
    end
    % print
    fprintf(file,' & %6d & %.1f & %.3f', ...
      dataset.num_clusters, ...
      dataset.mean_cluster_size, ...
      1-dataset.mean_conductance);
    if include_conductance_random
      fprintf(file,' & %.3f', 1-dataset.mean_conductance_random);
    end
    % end of row?
    if ~combined || id==numel(datasets) || ~isequal(dataset.display_name, datasets{id+1}.display_name)
      same_row = 0;
      fprintf(file,'\\\\\n');
    else
      same_row = 1;
    end
  end
  
  fprintf(file,'\\hlinebot\n');
  fprintf(file,'\\end{tabular}\n');
  fclose(file);
end

function out = pararrayfun_compat(n,f,xs,varargin)
  if exist('OCTAVE_VERSION','builtin')
    out = pararrayfun(n,f,xs,varargin{:});
  else
    opts = struct(varargin{:});
    if ~isfield(opts,'UniformOutput') opts.UniformOutput = true; end
    if opts.UniformOutput
      if isempty(xs)
        out = zeros(0,length(xs));
      else
        out = f(xs(1));
        out = [out,zeros(length(out),length(xs)-1)];
        for i=2:length(xs)
          out(:,i) = f(xs(i));
        end
      end
    else
      out = arrayfun(f,xs, 'UniformOutput',opts.UniformOutput);
    end
  end
end

